@@ -1,11 +1,10 @@
 import os
 import platform
 import struct
-import time
+import abc
 from itertools import count
 from multiprocessing import get_context
-
-from multiprocessing.reduction import ForkingPickler
+from pickle import dumps, loads
 from threading import BrokenBarrierError
 
 import trio
@@ -19,7 +18,7 @@ IDLE_TIMEOUT = 60 * 10
 _proc_counter = count()
 
 
-class WorkerProcBase:
+class WorkerProcBase(abc.ABC):
     def __init__(self, mp_context=get_context("spawn")):
         # It is almost possible to synchronize on the pipe alone but on Pypy
         # the _send_pipe doesn't raise the correct error on death. Anyway,
@@ -41,6 +40,7 @@ class WorkerProcBase:
     def _work(barrier, recv_pipe, send_pipe):  # pragma: no cover
 
         import inspect
+        import signal
         import outcome
 
         def coroutine_checker(fn, args):
@@ -55,6 +55,10 @@ class WorkerProcBase:
 
             return ret
 
+        # Intercept keyboard interrupts to avoid passing KeyboardInterrupt
+        # between processes. (Trio will take charge via cancellation.)
+        signal.signal(signal.SIGINT, signal.SIG_IGN)
+
         while True:
             try:
                 # Return value is party #, not whether awoken within timeout
@@ -63,13 +67,11 @@ class WorkerProcBase:
                 # Timeout waiting for job, so we can exit.
                 return
             # We got a job, and we are "woken"
-            fn, args = recv_pipe.recv()
+            fn, args = loads(recv_pipe.recv_bytes())
+            # Do the CPU bound work
             result = outcome.capture(coroutine_checker, fn, args)
-            # Tell the cache that we're done and available for a job
-            # Unlike the thread cache, it's impossible to deliver the
-            # result from the worker process. So shove it onto the queue
-            # and hope the receiver delivers the result and marks us idle
-            send_pipe.send(result)
+            # Send result and go back to idling
+            send_pipe.send_bytes(dumps(result, protocol=-1))
 
             del fn
             del args
@@ -79,8 +81,9 @@ class WorkerProcBase:
         # Neither this nor the child process should be waiting at this point
         assert not self._barrier.n_waiting, "Must first wake_up() the worker"
         try:
-            await self._send(ForkingPickler.dumps((sync_fn, args)))
-            result = ForkingPickler.loads(await self._recv())
+            await self._send(dumps((sync_fn, args), protocol=-1))
+            # noinspection PyTypeChecker
+            result = loads(await self._recv())
         except trio.EndOfChannel:
             # Likely the worker died while we were waiting on a pipe
             self.kill()  # NOTE: must reap zombie child elsewhere
@@ -109,31 +112,37 @@ class WorkerProcBase:
             # raise our own flavor of exception and reap child
             if self._proc.is_alive():  # pragma: no cover - rare race condition
                 self.kill()
-                self._proc.join()  # this will block for ms, but it should be rare
+                self._proc.join(1)  # this will block for ms, but it should be rare
+                if self._proc.is_alive():
+                    raise RuntimeError(f"{self._proc} alive after failed wakeup")
             raise BrokenWorkerError(f"{self._proc} died unexpectedly") from None
 
     def kill(self):
-        self._barrier.abort()
-        # race condition: if we kill while the proc has the underlying semaphore,
-        # we can deadlock it. Spinning on barrier n_waiting attribute doesn't
-        # work because they cheat while the _state is "draining" so spin on _count.
-        while self._barrier._count:
-            time.sleep(0.001)
-        try:
-            self._proc.kill()
-        except AttributeError:
-            self._proc.terminate()
+        # race condition: if we kill while the proc has the underlying
+        # semaphore, we can deadlock it, so make sure we hold it.
+        with self._barrier._cond:
+            self._barrier.abort()
+            try:
+                self._proc.kill()
+            except AttributeError:
+                # cpython 3.6 has an edge case where if this process holds
+                # the semaphore, a wait can timeout and raise an OSError.
+                self._proc.terminate()
 
-    def _rehabilitate_pipes(self):  # pragma: no cover
+    @abc.abstractmethod
+    async def wait(self):
         pass
 
-    async def _recv(self):  # pragma: no cover
+    @abc.abstractmethod
+    def _rehabilitate_pipes(self):
         pass
 
-    async def _send(self, buf):  # pragma: no cover
+    @abc.abstractmethod
+    async def _recv(self):
         pass
 
-    async def wait(self):  # pragma: no cover
+    @abc.abstractmethod
+    async def _send(self, buf):
         pass
 
 
@@ -148,8 +157,12 @@ class WindowsWorkerProc(WorkerProcBase):
 
         self._send_chan = PipeSendChannel(self._send_pipe.fileno())
         self._recv_chan = PipeReceiveChannel(self._recv_pipe.fileno())
-        self._send = self._send_chan.send
-        self._recv = self._recv_chan.receive
+
+    async def _recv(self):
+        return await self._recv_chan.receive()
+
+    async def _send(self, buf):
+        return await self._send_chan.send(buf)
 
     def __del__(self):
         # Avoid __del__ errors on cleanup: GH#174, GH#1767
@@ -236,12 +249,12 @@ class PosixWorkerProc(WorkerProcBase):
 class PypyWorkerProc(WorkerProcBase):
     async def run_sync(self, sync_fn, *args):
         async with trio.open_nursery() as nursery:
-            nursery.start_soon(self.child_monitor)
+            nursery.start_soon(self._child_monitor)
             result = await super().run_sync(sync_fn, *args)
             nursery.cancel_scope.cancel()
         return result
 
-    async def child_monitor(self):
+    async def _child_monitor(self):
         # If this worker dies, raise a catchable error...
         await self.wait()
         # but not if another error or cancel is incoming, those take priority!
